@@ -1,6 +1,7 @@
 #ifndef LOCK_PLATFORM_H_INCLUDED
 #define LOCK_PLATFORM_H_INCLUDED
 
+#include "xmlrpc-c/c_util.h"  /* For XMLRPC_DLLEXPORT */
 #include "xmlrpc-c/lock.h"
 
 /*
@@ -21,7 +22,7 @@ extern "C" {
 #endif
 
 XMLRPC_UTIL_EXPORTED
-lock *
+struct lock *
 xmlrpc_lock_create(void);
 
 #ifdef __cplusplus
