@@ -12,12 +12,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "xmlrpc_config.h"
 #include "bool.h"
 #include "int.h"
 #include "mallocvar.h"
 #include "xmlrpc-c/util_int.h"
 #include "xmlrpc-c/abyss.h"
-#ifdef _WIN32
+#if MSVCRT
   #include "socket_win.h"
 #else
   #include "socket_unix.h"
@@ -29,7 +30,7 @@
 static void
 socketOsInit(const char ** const errorP) {
 
-#ifdef _WIN32
+#if MSVCRT
     SocketWinInit(errorP);
 #else
     SocketUnixInit(errorP);
@@ -41,7 +42,7 @@ socketOsInit(const char ** const errorP) {
 static void
 socketOsTerm(void) {
 
-#ifdef _WIN32
+#if MSVCRT
     SocketWinTerm();
 #else
     SocketUnixTerm();
@@ -61,7 +62,7 @@ ChannelInit(const char ** const errorP) {
         ChannelTraceIsActive = (getenv("ABYSS_TRACE_CHANNEL") != NULL);
         if (ChannelTraceIsActive)
             fprintf(stderr, "Abyss channel layer will trace channel traffic "
-                    "due to ABYSS_TRACE_CHANNEL environment variable\n");
+                    "because of ABYSS_TRACE_CHANNEL environment variable\n");
     }
 }
 
