@@ -38,7 +38,7 @@ connJob(void * const userHandle) {
 
     (connectionP->job)(connectionP);
 
-    connectionP->finished = TRUE;
+    connectionP->finished = true;
         /* Note that if we are running in a forked process, setting
            connectionP->finished has no effect, because it's just our own
            copy of *connectionP.  In this case, Parent must update his own
@@ -68,7 +68,7 @@ connDone(TConn * const connectionP) {
     /* In the forked case, this is designed to run in the parent
        process after the child has terminated.
     */
-    connectionP->finished = TRUE;
+    connectionP->finished = true;
 
     if (connectionP->done)
         connectionP->done(connectionP);
@@ -82,7 +82,7 @@ static void
 threadDone(void * const userHandle) {
 
     TConn * const connectionP = userHandle;
-    
+
     connDone(connectionP);
 }
 
@@ -94,15 +94,15 @@ makeThread(TConn *             const connectionP,
            bool                const useSigchld,
            size_t              const jobStackSize,
            const char **       const errorP) {
-           
+
     switch (foregroundBackground) {
     case ABYSS_FOREGROUND:
-        connectionP->hasOwnThread = FALSE;
+        connectionP->hasOwnThread = false;
         *errorP = NULL;
         break;
     case ABYSS_BACKGROUND: {
         const char * error;
-        connectionP->hasOwnThread = TRUE;
+        connectionP->hasOwnThread = true;
         ThreadCreate(&connectionP->threadP, connectionP,
                      &connJob, &threadDone, useSigchld,
                      CONNJOB_STACK + jobStackSize,
@@ -117,7 +117,7 @@ makeThread(TConn *             const connectionP,
     } /* switch */
 }
 
-    
+
 
 void
 ConnCreate(TConn **            const connectionPP,
@@ -143,7 +143,7 @@ ConnCreate(TConn **            const connectionPP,
    function 'job' once.  Some connections can do that autonomously, as
    soon as the connection is created.  Others don't until Caller
    subsequently calls ConnProcess.  Some connections complete the
-   processing before ConnProcess return, while others may run the
+   processing before ConnProcess returns, while others may run the
    connection asynchronously to the creator, in the background, via a
    TThread thread.  'foregroundBackground' determines which.
 
@@ -169,7 +169,7 @@ ConnCreate(TConn **            const connectionPP,
         connectionP->buffer.b[0]  = '\0';
         connectionP->buffersize   = 0;
         connectionP->bufferpos    = 0;
-        connectionP->finished     = FALSE;
+        connectionP->finished     = false;
         connectionP->job          = job;
         connectionP->done         = done;
         connectionP->inbytes      = 0;
@@ -184,6 +184,22 @@ ConnCreate(TConn **            const connectionPP,
 
 
 
+uint32_t
+ConnBufferSpace(TConn * const connectionP) {
+/*-----------------------------------------------------------------------------
+   The amount of space left at the end of the connection buffer.
+
+   Note that the space before the read pointer (connectionP->bufferpos)
+   is also unused, but we don't consider that.
+-----------------------------------------------------------------------------*/
+    assert(connectionP->buffersize + 1 <= BUFFER_SIZE);
+
+    return BUFFER_SIZE - connectionP->buffersize - 1;
+        /* - 1 is because we reserve the last byte of the buffer for a NUL. */
+}
+
+
+
 bool
 ConnProcess(TConn * const connectionP) {
 /*----------------------------------------------------------------------------
@@ -208,7 +224,7 @@ ConnProcess(TConn * const connectionP) {
         /* No background thread.  We just handle it here while Caller waits. */
         (connectionP->job)(connectionP);
         connDone(connectionP);
-        retval = TRUE;
+        retval = true;
     }
     return retval;
 }
@@ -229,7 +245,7 @@ ConnWaitAndRelease(TConn * const connectionP) {
 
 bool
 ConnKill(TConn * const connectionP) {
-    connectionP->finished = TRUE;
+    connectionP->finished = true;
     return ThreadKill(connectionP->threadP);
 }
 
@@ -237,7 +253,14 @@ ConnKill(TConn * const connectionP) {
 
 void
 ConnReadInit(TConn * const connectionP) {
+/*-----------------------------------------------------------------------------
+   Prepare to read data from the channel into the connection buffer.
 
+   The connection buffer is not circular, so to make maximal room available
+   for this new data, we move everything now in the buffer (not yet
+   delivered to our client) down to the beginning of the buffer.  Caller
+   can then read from the end of that data through the end of the buffer.
+-----------------------------------------------------------------------------*/
     if (connectionP->buffersize > connectionP->bufferpos) {
         connectionP->buffersize -= connectionP->bufferpos;
         memmove(connectionP->buffer.b,
@@ -274,7 +297,7 @@ nextLineSize(const char * const string,
    string 'string', which is 'stringSize' characters long.
 
    'string' in not NUL-terminated.
-   
+
    A line begins at beginning of string or after a newline character and
    runs through the next newline character or end of string.  The line
    includes the newline character at the end, if any.
@@ -308,7 +331,7 @@ traceBuffer(const char *          const label,
         size_t const lineSize = nextLineSize(buffer_t, cursor, size);
         const char * const printableLine =
             xmlrpc_makePrintable_lp(&buffer_t[cursor], lineSize);
-        
+
         fprintf(stderr, "%s\n", printableLine);
 
         cursor += lineSize;
@@ -346,7 +369,7 @@ traceChannelWrite(TConn *      const connectionP,
                   const char * const buffer,
                   unsigned int const size,
                   bool         const failed) {
-    
+
     if (connectionP->trace) {
         const char * const label =
             failed ? "FAILED TO WRITE TO CHANNEL" : "WROTE TO CHANNEL";
@@ -356,43 +379,49 @@ traceChannelWrite(TConn *      const connectionP,
 
 
 
-static uint32_t
-bufferSpace(TConn * const connectionP) {
-    
-    return BUFFER_SIZE - connectionP->buffersize;
-}
-                    
-
-
 static void
 readFromChannel(TConn *       const connectionP,
                 bool *        const eofP,
                 const char ** const errorP) {
 /*----------------------------------------------------------------------------
-   Read some data from the channel of Connection *connectionP.
+   Read some data from the channel of Connection *connectionP into the
+   connection buffer.
 
-   Iff there is none available to read, return *eofP == true.
+   Iff there are no more bytes forthcoming on the channel, return *eofP ==
+   true.
+
+   Wait if necessary for at least one byte to arrive, or to learn that there
+   are no more bytes coming (EOF).  But don't wait for any more than that -
+   just read whatever is immediately available past that first byte.
+
+   Fail if the buffer is full.
 -----------------------------------------------------------------------------*/
     uint32_t bytesRead;
     bool readError;
 
-    ChannelRead(connectionP->channelP,
-                connectionP->buffer.b + connectionP->buffersize,
-                bufferSpace(connectionP) - 1,
-                &bytesRead, &readError);
+    assert(connectionP->buffersize <= BUFFER_SIZE);
 
-    if (readError)
-        xmlrpc_asprintf(errorP, "Error reading from channel");
+    if (BUFFER_SIZE - connectionP->buffersize < 2)
+        xmlrpc_asprintf(errorP, "Connection buffer full.");
     else {
-        *errorP = NULL;
-        if (bytesRead > 0) {
-            *eofP = FALSE;
-            traceChannelRead(connectionP, bytesRead);
-            connectionP->inbytes += bytesRead;
-            connectionP->buffersize += bytesRead;
-            connectionP->buffer.t[connectionP->buffersize] = '\0';
-        } else
-            *eofP = TRUE;
+        ChannelRead(connectionP->channelP,
+                    connectionP->buffer.b + connectionP->buffersize,
+                    BUFFER_SIZE - connectionP->buffersize - 1,
+                    &bytesRead, &readError);
+
+        if (readError)
+            xmlrpc_asprintf(errorP, "Error reading from channel");
+        else {
+            *errorP = NULL;
+            if (bytesRead > 0) {
+                *eofP = false;
+                traceChannelRead(connectionP, bytesRead);
+                connectionP->inbytes += bytesRead;
+                connectionP->buffersize += bytesRead;
+                connectionP->buffer.t[connectionP->buffersize] = '\0';
+            } else
+                *eofP = true;
+        }
     }
 }
 
@@ -457,6 +486,9 @@ ConnRead(TConn *       const connectionP,
    If 'eofP' is non-null, return *eofP == true, without reading anything, iff
    there will no more data forthcoming on the connection because client has
    closed the connection.  If 'eofP' is null, fail in that case.
+
+   Fail if the connection buffer is full.  (Caller can use ConnBufferSpace()
+   to find out beforehand if this is going to be a problem).
 -----------------------------------------------------------------------------*/
     uint32_t const timeoutMs = timeout * 1000;
 
@@ -464,15 +496,15 @@ ConnRead(TConn *       const connectionP,
         /* Arithmetic overflow */
         xmlrpc_asprintf(errorP, "Timeout value is too large");
     else {
-        bool const waitForRead  = TRUE;
-        bool const waitForWrite = FALSE;
+        bool const waitForRead  = true;
+        bool const waitForWrite = false;
 
         bool readyForRead;
         bool failed;
-            
+
         ChannelWait(connectionP->channelP, waitForRead, waitForWrite,
                     timeoutMs, &readyForRead, NULL, &failed);
-            
+
         if (failed)
             xmlrpc_asprintf(errorP,
                             "Wait for stuff to arrive from client failed.");
@@ -486,7 +518,7 @@ ConnRead(TConn *       const connectionP,
                 */
                 traceReadTimeout(connectionP, timeout);
                 *errorP = NULL;
-                eof = FALSE;
+                eof = false;
             }
             if (!*errorP)
                 dealWithReadTimeout(timedOutP, !readyForRead, timeout, errorP);
@@ -497,7 +529,7 @@ ConnRead(TConn *       const connectionP,
 }
 
 
-            
+
 bool
 ConnWrite(TConn *      const connectionP,
           const void * const buffer,
@@ -548,7 +580,7 @@ ConnWriteFromFile(TConn *       const connectionP,
 
     success = FileSeek(fileP, start, SEEK_SET);
     if (!success)
-        retval = FALSE;
+        retval = false;
     else {
         uint64_t const totalBytesToRead = last - start + 1;
         uint64_t bytesread;
@@ -559,19 +591,19 @@ ConnWriteFromFile(TConn *       const connectionP,
             uint64_t const bytesLeft     = totalBytesToRead - bytesread;
             uint64_t const bytesToRead64 = MIN(readChunkSize, bytesLeft);
             uint32_t const bytesToRead   = (uint32_t)bytesToRead64;
-            
+
             uint32_t bytesReadThisTime;
 
             assert(bytesToRead == bytesToRead64); /* readChunkSize is uint32 */
 
             bytesReadThisTime = FileRead(fileP, buffer, bytesToRead);
             bytesread += bytesReadThisTime;
-            
+
             if (bytesReadThisTime > 0)
                 ConnWrite(connectionP, buffer, bytesReadThisTime);
             else
                 break;
-            
+
             if (waittime > 0)
                 xmlrpc_millisecond_sleep(waittime);
         }
@@ -617,7 +649,7 @@ ConnFormatClientAddr(TConn *       const connectionP,
 **    documentation and/or other materials provided with the distribution.
 ** 3. The name of the author may not be used to endorse or promote products
 **    derived from this software without specific prior written permission.
-** 
+**
 ** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 ** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
