@@ -10,13 +10,6 @@
 #include <errno.h>
 #include <time.h>
 #include <fcntl.h>
-#ifdef _WIN32
-#  include <io.h>
-#else
-#  include <signal.h>
-#  include <sys/wait.h>
-#  include <grp.h>
-#endif
 
 #include "bool.h"
 #include "int.h"
@@ -48,25 +41,25 @@ addAuthCookie(xmlrpc_env * const envP,
               const char * const authCookie) {
 
     const char * cookieResponse;
-    
+
     xmlrpc_asprintf(&cookieResponse, "auth=%s", authCookie);
-    
+
     if (xmlrpc_strnomem(cookieResponse))
         xmlrpc_faultf(envP, "Insufficient memory to generate cookie "
                       "response header.");
     else {
         ResponseAddField(abyssSessionP, "Set-Cookie", cookieResponse);
-    
+
         xmlrpc_strfree(cookieResponse);
     }
-}   
-    
+}
 
 
-static void 
+
+static void
 sendResponse(xmlrpc_env *      const envP,
-             TSession *        const abyssSessionP, 
-             const char *      const body, 
+             TSession *        const abyssSessionP,
+             const char *      const body,
              size_t            const len,
              bool              const chunked,
              ResponseAccessCtl const accessControl) {
@@ -76,6 +69,8 @@ sendResponse(xmlrpc_env *      const envP,
 
    This is meant to run in the context of an Abyss URI handler for
    Abyss session 'abyssSessionP'.
+
+   'chunked' means to make it a chunked response if possible.
 -----------------------------------------------------------------------------*/
     const char * http_cookie = NULL;
         /* This used to set http_cookie to getenv("HTTP_COOKIE"), but
@@ -112,7 +107,7 @@ sendResponse(xmlrpc_env *      const envP,
         ResponseContentType(abyssSessionP, "text/xml; charset=utf-8");
         ResponseContentLength(abyssSessionP, abyssLen);
         ResponseAccessControl(abyssSessionP, accessControl);
-        
+
         ResponseWriteStart(abyssSessionP);
         ResponseWriteBody(abyssSessionP, body, abyssLen);
         ResponseWriteEnd(abyssSessionP);
@@ -139,7 +134,7 @@ sendResponse(xmlrpc_env *      const envP,
 
 
 static void
-sendError(TSession *   const abyssSessionP, 
+sendError(TSession *   const abyssSessionP,
           unsigned int const status,
           const char * const explanation) {
 /*----------------------------------------------------------------------------
@@ -205,13 +200,14 @@ getBody(xmlrpc_env *        const envP,
     body = xmlrpc_mem_block_new(envP, 0);
     if (!envP->fault_occurred) {
         size_t bytesRead;
-        const char * chunkPtr;
-        size_t chunkLen;
 
         bytesRead = 0;
 
         while (!envP->fault_occurred && bytesRead < contentSize) {
-            SessionGetReadData(abyssSessionP, contentSize - bytesRead, 
+            const char * chunkPtr;
+            size_t chunkLen;
+
+            SessionGetReadData(abyssSessionP, contentSize - bytesRead,
                                &chunkPtr, &chunkLen);
             bytesRead += chunkLen;
 
@@ -238,7 +234,7 @@ storeCookies(TSession *     const httpRequestP,
 -----------------------------------------------------------------------------*/
     const char * const cookie = RequestHeaderValue(httpRequestP, "cookie");
     if (cookie) {
-        /* 
+        /*
            Setting the value in an environment variable doesn't make
            any sense.  So for now, cookie code is disabled.
            -Bryan 04.10.03.
@@ -265,14 +261,14 @@ processContentLength(TSession *    const httpRequestP,
   don't want to figure out how to safely handle HTTP < 1.1 requests
   without it.
 -----------------------------------------------------------------------------*/
-    const char * const content_length = 
+    const char * const content_length =
         RequestHeaderValue(httpRequestP, "content-length");
 
     if (content_length == NULL) {
-        *missingP = TRUE;
+        *missingP = true;
         *errorP = NULL;
     } else {
-        *missingP = FALSE;
+        *missingP = false;
         *inputLenP = 0;  /* quiet compiler warning */
         if (content_length[0] == '\0')
             xmlrpc_asprintf(errorP, "The value in your content-length "
@@ -280,9 +276,9 @@ processContentLength(TSession *    const httpRequestP,
         else {
             unsigned long contentLengthValue;
             char * tail;
-        
+
             contentLengthValue = strtoul(content_length, &tail, 10);
-        
+
             if (*tail != '\0')
                 xmlrpc_asprintf(errorP, "There's non-numeric crap in "
                                 "the value of your content-length "
@@ -291,7 +287,7 @@ processContentLength(TSession *    const httpRequestP,
                 xmlrpc_asprintf(errorP, "According to your content-length "
                                 "HTTP header, your request is empty (zero "
                                 "length)");
-            else if ((unsigned long)(size_t)contentLengthValue 
+            else if ((unsigned long)(size_t)contentLengthValue
                      != contentLengthValue)
                 xmlrpc_asprintf(errorP, "According to your content-length "
                                 "HTTP header, your request is too big to "
@@ -309,7 +305,7 @@ processContentLength(TSession *    const httpRequestP,
 
 static void
 traceHandlerCalled(TSession * const abyssSessionP) {
-    
+
     const char * methodDesc;
     const TRequestInfo * requestInfoP;
 
@@ -353,10 +349,23 @@ processCall(TSession *            const abyssSessionP,
    Handle an RPC request.  This is an HTTP request that has the proper form
    to be an XML-RPC call.
 
-   The text of the call is available through the Abyss session
-   'abyssSessionP'.
+   We get the body of the request, which is the text of the call,
+   via the Abyss session 'abyssSessionP'.
 
    Its content length is 'contentSize' bytes.
+
+   We send the response to the request (which may contain the RPC response,
+   but may be an error indication) via the Abyss session 'abyssSessionP'.
+
+   We use 'xmlProcessor', with argument 'xmlProcessorArg' to execute the
+   RPC, i.e. turn the XML-RPC call into an XML-RPC response.
+
+   'wantChunk' means Caller wants the HTTP reponse chunked.
+
+   We use the Abyss session's memory pool for some memory allocations -
+   essentially those that aren't predictable because they depend upon the data
+   from the client.  We do this because the session's memory pool has a size
+   limit designed to keep the client from monopolizing the server's memory.
 -----------------------------------------------------------------------------*/
     xmlrpc_env env;
 
@@ -386,11 +395,11 @@ processCall(TSession *            const abyssSessionP,
                 &output);
             if (!env.fault_occurred) {
                 /* Send out the result. */
-                sendResponse(&env, abyssSessionP, 
+                sendResponse(&env, abyssSessionP,
                              XMLRPC_MEMBLOCK_CONTENTS(char, output),
                              XMLRPC_MEMBLOCK_SIZE(char, output),
                              wantChunk, accessControl);
-                
+
                 XMLRPC_MEMBLOCK_FREE(char, output);
             }
             XMLRPC_MEMBLOCK_FREE(char, body);
@@ -465,6 +474,11 @@ handleXmlRpcCallReq(TSession *           const abyssSessionP,
 
    Handle it by feeding the XML which is its content to 'xmlProcessor'
    along with argument 'xmlProcessorArg'.
+
+   (There doesn't seem to be any way 'xmlProcessor' could ever be anything but
+   'processXmlrpcCall' in xmlrpc_server_abyss.c (with 'xmlProcessorArg' being
+   the handle of a method registry), so I'm not sure why we didn't just put
+   that in this module and call it explicitly).
 -----------------------------------------------------------------------------*/
     /* We used to reject the call if content-type was not present and
        text/xml, on some security theory (a firewall may block text/xml with
@@ -484,7 +498,7 @@ handleXmlRpcCallReq(TSession *           const abyssSessionP,
         bool missing;
         size_t contentSize;
 
-        processContentLength(abyssSessionP, 
+        processContentLength(abyssSessionP,
                              &contentSize, &missing, &error);
         if (error) {
             sendError(abyssSessionP, 400, error);
@@ -510,7 +524,7 @@ handleXmlRpcOptionsReq(TSession *        const abyssSessionP,
                        ResponseAccessCtl const accessControl) {
 
     ResponseAddField(abyssSessionP, "Allow", "POST");
-    
+
     ResponseAccessControl(abyssSessionP, accessControl);
     ResponseContentLength(abyssSessionP, 0);
     ResponseStatus(abyssSessionP, 200);
@@ -530,7 +544,7 @@ xmlrpc_handleIfXmlrpcReq(void *        const handlerArg,
    call for this XML-RPC server, we handle it.  If it's not, we refuse
    it and Abyss can try some other handler.
 
-   Our return code is TRUE to mean we handled it; FALSE to mean we didn't.
+   We return *handledP true to mean we handled it; false to mean we didn't.
 
    Note that failing the request counts as handling it, and not handling
    it does not mean we failed it.
@@ -553,9 +567,9 @@ xmlrpc_handleIfXmlrpcReq(void *        const handlerArg,
         /* It's not for the path (e.g. "/RPC2") that we're supposed to
            handle.
         */
-        *handledP = FALSE;
+        *handledP = false;
     else {
-        *handledP = TRUE;
+        *handledP = true;
 
         switch (requestInfoP->method) {
         case m_post:
@@ -621,7 +635,7 @@ xmlrpc_serverAbyssDefaultUriHandler(TSession * const sessionP) {
 
     xmlrpc_strfree(explanation);
 
-    return TRUE;
+    return true;
 }
 
 
