@@ -37,7 +37,7 @@ using girerr::error;
 #include "xmlrpc-c/base.h"
 #include "xmlrpc-c/base_int.h"
 #include "xmlrpc-c/string_int.h"
-#include "env_wrap.hpp"
+#include "xmlrpc-c/env_wrap.hpp"
 
 #include "xmlrpc-c/base.hpp"
 
@@ -58,23 +58,23 @@ throwIfError(env_wrap const& env) {
 class cDatetimeValueWrapper {
 public:
     xmlrpc_value * valueP;
-    
+
     cDatetimeValueWrapper(xmlrpc_datetime const cppvalue) {
         env_wrap env;
-        
+
         this->valueP = xmlrpc_datetime_new(&env.env_c, cppvalue);
         throwIfError(env);
     }
     cDatetimeValueWrapper(time_t const cppvalue) {
         env_wrap env;
-        
+
         this->valueP = xmlrpc_datetime_new_sec(&env.env_c, cppvalue);
         throwIfError(env);
     }
 #if XMLRPC_HAVE_TIMEVAL
     cDatetimeValueWrapper(struct timeval const cppvalue) {
         env_wrap env;
-        
+
         this->valueP = xmlrpc_datetime_new_timeval(&env.env_c, cppvalue);
         throwIfError(env);
     }
@@ -82,7 +82,7 @@ public:
 #if XMLRPC_HAVE_TIMESPEC
     cDatetimeValueWrapper(struct timespec const cppvalue) {
         env_wrap env;
-        
+
         this->valueP = xmlrpc_datetime_new_timespec(&env.env_c, cppvalue);
         throwIfError(env);
     }
@@ -235,7 +235,7 @@ value::addToCStruct(xmlrpc_value * const structP,
 
 
 
-value::type_t 
+value::type_t
 value::type() const {
 
     this->validateInstantiated();
@@ -270,10 +270,10 @@ value_int::value_int(int const cppvalue) {
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper(int const cppvalue) {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_int_new(&env.env_c, cppvalue);
             throwIfError(env);
         }
@@ -281,9 +281,9 @@ value_int::value_int(int const cppvalue) {
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper(cppvalue);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -328,10 +328,10 @@ value_double::value_double(double const cppvalue) {
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper(double const cppvalue) {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_double_new(&env.env_c, cppvalue);
             throwIfError(env);
         }
@@ -339,7 +339,7 @@ value_double::value_double(double const cppvalue) {
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     this->instantiate(cWrapper(cppvalue).valueP);
 }
 
@@ -385,10 +385,10 @@ value_boolean::value_boolean(bool const cppvalue) {
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper(xmlrpc_bool const cppvalue) {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_bool_new(&env.env_c, cppvalue);
             throwIfError(env);
         }
@@ -396,9 +396,9 @@ value_boolean::value_boolean(bool const cppvalue) {
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper(cppvalue);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -444,10 +444,10 @@ value_datetime::value_datetime(string const cppvalue) {
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper(string const cppvalue) {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_datetime_new_str(&env.env_c,
                                                    cppvalue.c_str());
             throwIfError(env);
@@ -456,9 +456,9 @@ value_datetime::value_datetime(string const cppvalue) {
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper(cppvalue);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -467,7 +467,7 @@ value_datetime::value_datetime(string const cppvalue) {
 value_datetime::value_datetime(xmlrpc_datetime const cppvalue) {
 
     cDatetimeValueWrapper wrapper(cppvalue);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -476,7 +476,7 @@ value_datetime::value_datetime(xmlrpc_datetime const cppvalue) {
 value_datetime::value_datetime(time_t const cppvalue) {
 
     cDatetimeValueWrapper wrapper(cppvalue);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -614,11 +614,11 @@ value_datetime::iso8601Value() const {
 class cNewStringWrapper {
 public:
     xmlrpc_value * valueP;
-        
+
     cNewStringWrapper(string               const cppvalue,
                       value_string::nlCode const nlCode) {
         env_wrap env;
-            
+
         switch (nlCode) {
         case value_string::nlCode_all:
             this->valueP = xmlrpc_string_new_lp(&env.env_c,
@@ -641,14 +641,14 @@ public:
         xmlrpc_DECREF(this->valueP);
     }
 };
-    
+
 
 
 value_string::value_string(std::string          const& cppvalue,
                            value_string::nlCode const  nlCode) {
-    
+
     cNewStringWrapper wrapper(cppvalue, nlCode);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -657,12 +657,12 @@ value_string::value_string(std::string          const& cppvalue,
 value_string::value_string(std::string const& cppvalue) {
 
     cNewStringWrapper wrapper(cppvalue, nlCode_all);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
 
-    
+
 value_string::value_string(xmlrpc_c::value const baseValue) {
 
     if (baseValue.type() != xmlrpc_c::value::TYPE_STRING)
@@ -683,7 +683,7 @@ value_string::crlfValue() const {
         size_t length;
         cWrapper(xmlrpc_value * valueP) {
             env_wrap env;
-            
+
             xmlrpc_read_string_lp_crlf(&env.env_c, valueP, &length, &str);
             throwIfError(env);
         }
@@ -691,7 +691,7 @@ value_string::crlfValue() const {
             free((char*)str);
         }
     };
-    
+
     this->validateInstantiated();
 
     cWrapper wrapper(this->cValueP);
@@ -701,6 +701,27 @@ value_string::crlfValue() const {
 
 
 
+void
+value_string::validate() const {
+/*----------------------------------------------------------------------------
+   Throw an error if the value contains non-XML characters, so that if it
+   were serialized into a call or response, it would not be valid XML, and
+   therefore not proper XML-RPC.
+
+   Note that the object must have the ability to have this property (i.e. it
+   is possible to construct an object that fails this validation) because of
+   the poor original specification of XML-RPC.  Some recipients of XML-RPC
+   don't mind it being invalid XML.
+-----------------------------------------------------------------------------*/
+    env_wrap env;
+
+    xmlrpc_string_validate(&env.env_c, this->cValueP);
+
+    throwIfError(env);
+}
+
+
+
 value_string::operator string() const {
 
     this->validateInstantiated();
@@ -726,11 +747,11 @@ value_bytestring::value_bytestring(
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper(vector<unsigned char> const& cppvalue) {
             env_wrap env;
-            
-            this->valueP = 
+
+            this->valueP =
                 xmlrpc_base64_new(&env.env_c, cppvalue.size(), &cppvalue[0]);
             throwIfError(env);
         }
@@ -738,9 +759,9 @@ value_bytestring::value_bytestring(
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper(cppvalue);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -775,12 +796,12 @@ value_bytestring::vectorUcharValue() const {
             free((void*)contents);
         }
     };
-    
+
     this->validateInstantiated();
 
     cWrapper wrapper(this->cValueP);
 
-    return vector<unsigned char>(&wrapper.contents[0], 
+    return vector<unsigned char>(&wrapper.contents[0],
                                  &wrapper.contents[wrapper.length]);
 }
 
@@ -811,14 +832,14 @@ value_bytestring::length() const {
 
 
 value_array::value_array(vector<xmlrpc_c::value> const& cppvalue) {
-    
+
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper() {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_array_new(&env.env_c);
             throwIfError(env);
         }
@@ -826,9 +847,9 @@ value_array::value_array(vector<xmlrpc_c::value> const& cppvalue) {
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper;
-    
+
     vector<xmlrpc_c::value>::const_iterator i;
     for (i = cppvalue.begin(); i != cppvalue.end(); ++i)
         i->appendToCArray(wrapper.valueP);
@@ -860,9 +881,9 @@ value_array::vectorValueValue() const {
 
     arraySize = xmlrpc_array_size(&env.env_c, this->cValueP);
     throwIfError(env);
-    
+
     vector<xmlrpc_c::value> retval(arraySize);
-    
+
     for (unsigned int i = 0; i < arraySize; ++i) {
 
         class cWrapper {
@@ -874,7 +895,7 @@ value_array::vectorValueValue() const {
                 env_wrap env;
 
                 xmlrpc_array_read_item(&env.env_c, arrayP, index, &valueP);
-                
+
                 throwIfError(env);
             }
             ~cWrapper() {
@@ -910,7 +931,7 @@ value_array::size() const {
 
     arraySize = xmlrpc_array_size(&env.env_c, this->cValueP);
     throwIfError(env);
-    
+
     return arraySize;
 }
 
@@ -922,10 +943,10 @@ value_struct::value_struct(
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper() {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_struct_new(&env.env_c);
             throwIfError(env);
         }
@@ -933,7 +954,7 @@ value_struct::value_struct(
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper;
 
     map<string, xmlrpc_c::value>::const_iterator i;
@@ -942,7 +963,7 @@ value_struct::value_struct(
         string          mapkey(i->first);
         mapvalue.addToCStruct(wrapper.valueP, mapkey);
     }
-    
+
     this->instantiate(wrapper.valueP);
 }
 
@@ -968,9 +989,9 @@ value_struct::operator map<string, xmlrpc_c::value>() const {
 
     structSize = xmlrpc_struct_size(&env.env_c, this->cValueP);
     throwIfError(env);
-    
+
     map<string, xmlrpc_c::value> retval;
-    
+
     for (unsigned int i = 0; i < structSize; ++i) {
         class cMemberWrapper {
         public:
@@ -981,10 +1002,10 @@ value_struct::operator map<string, xmlrpc_c::value>() const {
                            unsigned int   const index) {
 
                 env_wrap env;
-            
-                xmlrpc_struct_read_member(&env.env_c, structP, index, 
+
+                xmlrpc_struct_read_member(&env.env_c, structP, index,
                                           &keyP, &valueP);
-                
+
                 throwIfError(env);
             }
             ~cMemberWrapper() {
@@ -1016,14 +1037,14 @@ value_struct::cvalue() const {
 
 
 value_nil::value_nil() {
-    
+
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper() {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_nil_new(&env.env_c);
             throwIfError(env);
         }
@@ -1031,12 +1052,12 @@ value_nil::value_nil() {
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper;
-    
+
     this->instantiate(wrapper.valueP);
 }
-    
+
 
 
 value_nil::value_nil(xmlrpc_c::value const baseValue) {
@@ -1063,10 +1084,10 @@ value_i8::value_i8(xmlrpc_int64 const cppvalue) {
     class cWrapper {
     public:
         xmlrpc_value * valueP;
-        
+
         cWrapper(xmlrpc_int64 const cppvalue) {
             env_wrap env;
-            
+
             this->valueP = xmlrpc_i8_new(&env.env_c, cppvalue);
             throwIfError(env);
         }
@@ -1074,9 +1095,9 @@ value_i8::value_i8(xmlrpc_int64 const cppvalue) {
             xmlrpc_DECREF(this->valueP);
         }
     };
-    
+
     cWrapper wrapper(cppvalue);
-    
+
     this->instantiate(wrapper.valueP);
 }
 
