@@ -2,10 +2,12 @@
 
 #include "xmlrpc_config.h"
 
+#include <assert.h>
 #include <stddef.h>
 #include <stdlib.h>
 #include <stdarg.h>
 #include <string.h>
+#include <math.h>
 
 #include "bool.h"
 #include "mallocvar.h"
@@ -54,7 +56,7 @@ destroyValue(xmlrpc_value * const valueP) {
     switch (valueP->_type) {
     case XMLRPC_TYPE_INT:
         break;
-        
+
     case XMLRPC_TYPE_BOOL:
         break;
 
@@ -68,15 +70,15 @@ destroyValue(xmlrpc_value * const valueP) {
     case XMLRPC_TYPE_STRING:
         xmlrpc_destroyString(valueP);
         break;
-        
+
     case XMLRPC_TYPE_BASE64:
-        xmlrpc_mem_block_clean(&valueP->_block);
+        xmlrpc_mem_block_free(valueP->blockP);
         break;
 
     case XMLRPC_TYPE_ARRAY:
         xmlrpc_destroyArrayContents(valueP);
         break;
-        
+
     case XMLRPC_TYPE_STRUCT:
         xmlrpc_destroyStruct(valueP);
         break;
@@ -93,6 +95,7 @@ destroyValue(xmlrpc_value * const valueP) {
 
     case XMLRPC_TYPE_DEAD:
         XMLRPC_ASSERT(false); /* Can't happen, per entry conditions */
+        break;
 
     default:
         XMLRPC_ASSERT(false); /* There are no other possible values */
@@ -117,7 +120,7 @@ destroyValue(xmlrpc_value * const valueP) {
   charge of destroying values when their reference count reaches zero.
 ============================================================================*/
 
-void 
+void
 xmlrpc_INCREF (xmlrpc_value * const valueP) {
 
     XMLRPC_ASSERT_VALUE_OK(valueP);
@@ -125,7 +128,7 @@ xmlrpc_INCREF (xmlrpc_value * const valueP) {
     valueP->lockP->acquire(valueP->lockP);
 
     XMLRPC_ASSERT(valueP->refcount > 0);
-    
+
     ++valueP->refcount;
 
     valueP->lockP->release(valueP->lockP);
@@ -133,7 +136,7 @@ xmlrpc_INCREF (xmlrpc_value * const valueP) {
 
 
 
-void 
+void
 xmlrpc_DECREF (xmlrpc_value * const valueP) {
 
     bool died;
@@ -193,7 +196,7 @@ validateType(xmlrpc_env *         const envP,
     if (valueP->_type != expectedType) {
         xmlrpc_env_set_fault_formatted(
             envP, XMLRPC_TYPE_ERROR, "Value of type %s supplied where "
-            "type %s was expected.", 
+            "type %s was expected.",
             xmlrpc_type_name(valueP->_type), xmlrpc_type_name(expectedType));
     }
 }
@@ -209,7 +212,7 @@ validateType(xmlrpc_env *         const envP,
   and xmlrpc_struct.c.
 =========================================================================*/
 
-void 
+void
 xmlrpc_read_int(xmlrpc_env *         const envP,
                 const xmlrpc_value * const valueP,
                 xmlrpc_int32 *       const intValueP) {
@@ -237,7 +240,7 @@ void
 xmlrpc_read_double(xmlrpc_env *         const envP,
                    const xmlrpc_value * const valueP,
                    xmlrpc_double *      const doubleValueP) {
-    
+
     validateType(envP, valueP, XMLRPC_TYPE_DOUBLE);
     if (!envP->fault_occurred)
         *doubleValueP = valueP->_value.d;
@@ -260,10 +263,10 @@ xmlrpc_read_base64(xmlrpc_env *           const envP,
 
     validateType(envP, valueP, XMLRPC_TYPE_BASE64);
     if (!envP->fault_occurred) {
-        size_t const size = 
-            XMLRPC_MEMBLOCK_SIZE(char, &valueP->_block);
-        const char * const contents = 
-            XMLRPC_MEMBLOCK_CONTENTS(char, &valueP->_block);
+        size_t const size =
+            XMLRPC_MEMBLOCK_SIZE(char, valueP->blockP);
+        const char * const contents =
+            XMLRPC_MEMBLOCK_CONTENTS(char, valueP->blockP);
 
         char * byteStringValue;
 
@@ -291,9 +294,9 @@ xmlrpc_read_base64_old(xmlrpc_env *           const envP,
     validateType(envP, valueP, XMLRPC_TYPE_BASE64);
     if (!envP->fault_occurred) {
         *lengthP =
-            XMLRPC_MEMBLOCK_SIZE(char, &valueP->_block);
+            XMLRPC_MEMBLOCK_SIZE(char, valueP->blockP);
         *byteStringValueP = (const unsigned char *)
-            XMLRPC_MEMBLOCK_CONTENTS(char, &valueP->_block);
+            XMLRPC_MEMBLOCK_CONTENTS(char, valueP->blockP);
     }
 }
 
@@ -306,7 +309,7 @@ xmlrpc_read_base64_size(xmlrpc_env *           const envP,
 
     validateType(envP, valueP, XMLRPC_TYPE_BASE64);
     if (!envP->fault_occurred)
-        *lengthP = XMLRPC_MEMBLOCK_SIZE(char, &valueP->_block);
+        *lengthP = XMLRPC_MEMBLOCK_SIZE(char, valueP->blockP);
 }
 
 
@@ -336,7 +339,7 @@ xmlrpc_read_nil(xmlrpc_env *   const envP,
 
 
 
-void 
+void
 xmlrpc_read_i8(xmlrpc_env *         const envP,
                const xmlrpc_value * const valueP,
                xmlrpc_int64 *       const intValueP) {
@@ -377,6 +380,11 @@ xmlrpc_createXmlrpcValue(xmlrpc_env *    const envP,
                           "xmlrpc_value");
         else
             valP->refcount = 1;
+
+        if (envP->fault_occurred) {
+            free(valP);
+            valP = NULL;
+        }
     }
     *valPP = valP;
 }
@@ -384,7 +392,45 @@ xmlrpc_createXmlrpcValue(xmlrpc_env *    const envP,
 
 
 xmlrpc_value *
-xmlrpc_int_new(xmlrpc_env * const envP, 
+xmlrpc_value_new(xmlrpc_env *   const envP,
+                 xmlrpc_value * const sourceValP) {
+
+    switch (sourceValP->_type) {
+    case XMLRPC_TYPE_INT:
+        return xmlrpc_int_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_I8:
+        return xmlrpc_i8_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_BOOL:
+        return xmlrpc_bool_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_DOUBLE:
+        return xmlrpc_double_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_DATETIME:
+        return xmlrpc_datetime_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_STRING:
+        return xmlrpc_string_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_BASE64:
+        return xmlrpc_base64_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_ARRAY:
+        return xmlrpc_array_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_STRUCT:
+        return xmlrpc_struct_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_C_PTR:
+        return xmlrpc_cptr_new_value(envP, sourceValP);
+    case XMLRPC_TYPE_NIL:
+        return xmlrpc_nil_new(envP);
+    case XMLRPC_TYPE_DEAD:
+        xmlrpc_faultf(envP, "Attempt to copy a dead xmlrpc_value");
+        return NULL;
+    }
+    assert(false);  /* All cases are handled above */
+
+    return NULL; /* Quiet compiler warning */
+}
+
+
+
+xmlrpc_value *
+xmlrpc_int_new(xmlrpc_env * const envP,
                xmlrpc_int32 const value) {
 
     xmlrpc_value * valP;
@@ -401,7 +447,26 @@ xmlrpc_int_new(xmlrpc_env * const envP,
 
 
 xmlrpc_value *
-xmlrpc_i8_new(xmlrpc_env * const envP, 
+xmlrpc_int_new_value(xmlrpc_env *   const envP,
+                     xmlrpc_value * const valueP) {
+
+    xmlrpc_value * retval;
+
+    if (valueP->_type != XMLRPC_TYPE_INT) {
+        xmlrpc_env_set_fault_formatted(envP, XMLRPC_TYPE_ERROR,
+                                       "Value is not an integer.  "
+                                       "It is type #%d", valueP->_type);
+        retval = NULL;
+    } else
+        retval = xmlrpc_int_new(envP, valueP->_value.i);
+
+    return retval;
+}
+
+
+
+xmlrpc_value *
+xmlrpc_i8_new(xmlrpc_env * const envP,
               xmlrpc_int64 const value) {
 
     xmlrpc_value * valP;
@@ -418,7 +483,26 @@ xmlrpc_i8_new(xmlrpc_env * const envP,
 
 
 xmlrpc_value *
-xmlrpc_bool_new(xmlrpc_env * const envP, 
+xmlrpc_i8_new_value(xmlrpc_env *   const envP,
+                    xmlrpc_value * const valueP) {
+
+    xmlrpc_value * retval;
+
+    if (valueP->_type != XMLRPC_TYPE_I8) {
+        xmlrpc_env_set_fault_formatted(envP, XMLRPC_TYPE_ERROR,
+                                       "Value is not a 64-bit integer.  "
+                                       "It is type #%d", valueP->_type);
+        retval = NULL;
+    } else
+        retval = xmlrpc_i8_new(envP, valueP->_value.i8);
+
+    return retval;
+}
+
+
+
+xmlrpc_value *
+xmlrpc_bool_new(xmlrpc_env * const envP,
                 xmlrpc_bool  const value) {
 
     xmlrpc_value * valP;
@@ -435,16 +519,41 @@ xmlrpc_bool_new(xmlrpc_env * const envP,
 
 
 xmlrpc_value *
-xmlrpc_double_new(xmlrpc_env * const envP, 
+xmlrpc_bool_new_value(xmlrpc_env *   const envP,
+                      xmlrpc_value * const valueP) {
+
+    xmlrpc_value * retval;
+
+    if (valueP->_type != XMLRPC_TYPE_BOOL) {
+        xmlrpc_env_set_fault_formatted(envP, XMLRPC_TYPE_ERROR,
+                                       "Value is not a boolean.  "
+                                       "It is type #%d", valueP->_type);
+        retval = NULL;
+    } else
+        retval = xmlrpc_bool_new(envP, valueP->_value.b);
+
+    return retval;
+}
+
+
+
+xmlrpc_value *
+xmlrpc_double_new(xmlrpc_env * const envP,
                   double       const value) {
 
     xmlrpc_value * valP;
 
-    xmlrpc_createXmlrpcValue(envP, &valP);
+    if (!XMLRPC_FINITE(value)) {
+        xmlrpc_faultf(envP, "Value is not a finite number, "
+                      "so cannot be represented in XML-RPC");
+        valP = NULL;
+    } else {
+        xmlrpc_createXmlrpcValue(envP, &valP);
 
-    if (!envP->fault_occurred) {
-        valP->_type = XMLRPC_TYPE_DOUBLE;
-        valP->_value.d = value;
+        if (!envP->fault_occurred) {
+            valP->_type = XMLRPC_TYPE_DOUBLE;
+            valP->_value.d = value;
+        }
     }
     return valP;
 }
@@ -452,7 +561,27 @@ xmlrpc_double_new(xmlrpc_env * const envP,
 
 
 xmlrpc_value *
-xmlrpc_base64_new(xmlrpc_env *          const envP, 
+xmlrpc_double_new_value(xmlrpc_env *   const envP,
+                        xmlrpc_value * const valueP) {
+
+    xmlrpc_value * retval;
+
+    if (valueP->_type != XMLRPC_TYPE_DOUBLE) {
+        xmlrpc_env_set_fault_formatted(
+            envP, XMLRPC_TYPE_ERROR,
+            "Value is not a floating point number.  "
+            "It is type #%d", valueP->_type);
+        retval = NULL;
+    } else
+        retval = xmlrpc_double_new(envP, valueP->_value.d);
+
+    return retval;
+}
+
+
+
+xmlrpc_value *
+xmlrpc_base64_new(xmlrpc_env *          const envP,
                   size_t                const length,
                   const unsigned char * const value) {
 
@@ -463,10 +592,10 @@ xmlrpc_base64_new(xmlrpc_env *          const envP,
     if (!envP->fault_occurred) {
         valP->_type = XMLRPC_TYPE_BASE64;
 
-        xmlrpc_mem_block_init(envP, &valP->_block, length);
+        valP->blockP = xmlrpc_mem_block_new(envP, length);
         if (!envP->fault_occurred) {
-            char * const contents = 
-                xmlrpc_mem_block_contents(&valP->_block);
+            char * const contents =
+                xmlrpc_mem_block_contents(valP->blockP);
             memcpy(contents, value, length);
         }
         if (envP->fault_occurred)
@@ -477,6 +606,27 @@ xmlrpc_base64_new(xmlrpc_env *          const envP,
 
 
 
+xmlrpc_value *
+xmlrpc_base64_new_value(xmlrpc_env *   const envP,
+                        xmlrpc_value * const valueP) {
+
+    xmlrpc_value * retval;
+
+    if (valueP->_type != XMLRPC_TYPE_BASE64) {
+        xmlrpc_env_set_fault_formatted(envP, XMLRPC_TYPE_ERROR,
+                                       "Value is not a datetime.  "
+                                       "It is type #%d", valueP->_type);
+        retval = NULL;
+    } else
+        retval = xmlrpc_base64_new(envP,
+                                   xmlrpc_mem_block_size(valueP->blockP),
+                                   xmlrpc_mem_block_contents(valueP->blockP));
+
+    return retval;
+}
+
+
+
 /* array stuff is in xmlrpc_array.c */
 
 
@@ -511,6 +661,27 @@ xmlrpc_cptr_new_dtor(xmlrpc_env *        const envP,
 
 
 
+xmlrpc_value *
+xmlrpc_cptr_new_value(xmlrpc_env *   const envP,
+                      xmlrpc_value * const valueP) {
+
+    xmlrpc_value * retval;
+
+    if (valueP->_type != XMLRPC_TYPE_C_PTR) {
+        xmlrpc_env_set_fault_formatted(envP, XMLRPC_TYPE_ERROR,
+                                       "Value is not a C poitner.  "
+                                       "It is type #%d", valueP->_type);
+        retval = NULL;
+    } else
+        retval = xmlrpc_cptr_new_dtor(envP,
+                                      valueP->_value.cptr.objectP,
+                                      valueP->_value.cptr.dtor,
+                                      valueP->_value.cptr.dtorContext);
+    return retval;
+}
+
+
+
 xmlrpc_value *
 xmlrpc_nil_new(xmlrpc_env *    const envP) {
     xmlrpc_value * valP;
@@ -536,8 +707,8 @@ xmlrpc_nil_new(xmlrpc_env *    const envP) {
 **    notice, this list of conditions and the following disclaimer in the
 **    documentation and/or other materials provided with the distribution.
 ** 3. The name of the author may not be used to endorse or promote products
-**    derived from this software without specific prior written permission. 
-**  
+**    derived from this software without specific prior written permission.
+**
 ** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 ** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
