@@ -114,7 +114,7 @@ struct structValDecomp {
 struct arrayDecomp {
     unsigned int itemCnt;
     bool ignoreExcess;
-        /* If there are more that 'itemCnt' items in the array, just
+        /* If there are more than 'itemCnt' items in the array, just
            extract the first 'itemCnt' and ignore the rest, rather than
            fail the decomposition.
         */
@@ -217,7 +217,11 @@ releaseDecomposition(const struct decompTreeNode * const decompRootP) {
         xmlrpc_strfree(*decompRootP->store.Tstring.valueP);
         break;
     case 'w':
+#if HAVE_UNICODE_WCHAR
         free((void*)*decompRootP->store.TwideString.valueP);
+#else
+	XMLRPC_ASSERT(false);
+#endif
         break;
     case '6':
         free((void*)*decompRootP->store.TbitString.valueP);
