@@ -11,8 +11,8 @@
 **    notice, this list of conditions and the following disclaimer in the
 **    documentation and/or other materials provided with the distribution.
 ** 3. The name of the author may not be used to endorse or promote products
-**    derived from this software without specific prior written permission. 
-**  
+**    derived from this software without specific prior written permission.
+**
 ** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 ** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
@@ -43,13 +43,14 @@
 #include "xmlrpc-c/base.h"
 #include "xmlrpc-c/base_int.h"
 #include "xmlrpc-c/string_int.h"
-#include "xmlrpc-c/xmlparser.h"
+
+#include "xmlparser.h"
 
 struct _xml_element {
     xml_element * parentP;
     const char * name;
-    xmlrpc_mem_block cdata;    /* char */
-    xmlrpc_mem_block children; /* xml_element* */
+    xmlrpc_mem_block * cdataP;    /* char */
+    xmlrpc_mem_block * childrenP; /* xml_element* */
 };
 
 #define XMLRPC_ASSERT_ELEM_OK(elem) \
@@ -96,51 +97,40 @@ xmlElementNew(xmlrpc_env * const envP,
 -----------------------------------------------------------------------------*/
 
     xml_element * retval;
-    bool nameIsValid;
-    bool cdataIsValid;
-    bool childrenAreValid;
 
     XMLRPC_ASSERT_ENV_OK(envP);
     assert(name != NULL);
 
-    /* Set up our error-handling preconditions. */
-    retval = NULL;
-    nameIsValid = cdataIsValid = childrenAreValid = false;
-
     MALLOCVAR(retval);
-    XMLRPC_FAIL_IF_NULL(retval, envP, XMLRPC_INTERNAL_ERROR,
-                        "Couldn't allocate memory for XML element");
 
-    retval->parentP = NULL;
-    
-    /* Copy over the element name. */
-    retval->name = strdup(name);
-    XMLRPC_FAIL_IF_NULL(retval->name, envP, XMLRPC_INTERNAL_ERROR,
-                        "Couldn't allocate memory for XML element");
-    nameIsValid = true;
+    if (!retval)
+        xmlrpc_faultf(envP, "Couldn't allocate memory for XML element");
+    else {
+        retval->parentP = NULL;
 
-    /* Initialize a block to hold our CDATA. */
-    XMLRPC_TYPED_MEM_BLOCK_INIT(char, envP, &retval->cdata, 0);
-    XMLRPC_FAIL_IF_FAULT(envP);
-    cdataIsValid = true;
+        retval->name = strdup(name);
 
-    /* Initialize a block to hold our child elements. */
-    XMLRPC_TYPED_MEM_BLOCK_INIT(xml_element *, envP, &retval->children, 0);
-    XMLRPC_FAIL_IF_FAULT(envP);
-    childrenAreValid = true;
+        if (!retval->name)
+            xmlrpc_faultf(envP, "Couldn't allocate memory for name field "
+                          "of XML element");
+        else {
+            retval->cdataP = XMLRPC_MEMBLOCK_NEW(char, envP, 0);
 
-cleanup:
-    if (envP->fault_occurred) {
-        if (retval) {
-            if (nameIsValid)
+            if (!envP->fault_occurred) {
+                retval->childrenP =
+                    XMLRPC_MEMBLOCK_NEW(xml_element, envP, 0);
+                if (!envP->fault_occurred) {
+                    if (envP->fault_occurred)
+                        xmlrpc_mem_block_free(retval->childrenP);
+                }
+                if (envP->fault_occurred)
+                    xmlrpc_mem_block_free(retval->cdataP);
+            }
+            if (envP->fault_occurred)
                 xmlrpc_strfree(retval->name);
-            if (cdataIsValid)
-                xmlrpc_mem_block_clean(&retval->cdata);
-            if (childrenAreValid)
-                xmlrpc_mem_block_clean(&retval->children);
-            free(retval);
         }
-        retval = NULL;
+        if (envP->fault_occurred)
+            free(retval);
     }
     return retval;
 }
@@ -150,9 +140,8 @@ cleanup:
 void
 xml_element_free(xml_element * const elemP) {
 /*----------------------------------------------------------------------------
-  Blow away an existing element & all of its child elements.
+  Blow away an existing element and all of its child elements.
 -----------------------------------------------------------------------------*/
-    xmlrpc_mem_block * children;
     unsigned int size;
     unsigned int i;
     xml_element ** contents;
@@ -161,16 +150,15 @@ xml_element_free(xml_element * const elemP) {
 
     xmlrpc_strfree(elemP->name);
     elemP->name = XMLRPC_BAD_POINTER;
-    xmlrpc_mem_block_clean(&elemP->cdata);
+    xmlrpc_mem_block_free(elemP->cdataP);
 
     /* Deallocate all of our children recursively. */
-    children = &elemP->children;
-    contents = XMLRPC_TYPED_MEM_BLOCK_CONTENTS(xml_element *, children);
-    size = XMLRPC_TYPED_MEM_BLOCK_SIZE(xml_element *, children);
+    contents = XMLRPC_MEMBLOCK_CONTENTS(xml_element *, elemP->childrenP);
+    size = XMLRPC_MEMBLOCK_SIZE(xml_element *, elemP->childrenP);
     for (i = 0; i < size; ++i)
         xml_element_free(contents[i]);
 
-    xmlrpc_mem_block_clean(&elemP->children);
+    xmlrpc_mem_block_free(elemP->childrenP);
 
     free(elemP);
 }
@@ -192,28 +180,28 @@ xml_element_name(const xml_element * const elemP) {
 }
 
 size_t
-xml_element_cdata_size(xml_element * const elemP) {
+xml_element_cdata_size(const xml_element * const elemP) {
     /* The result of this function is NOT VALID until the end_element handler
        has been called!
     */
     XMLRPC_ASSERT_ELEM_OK(elemP);
-    return XMLRPC_TYPED_MEM_BLOCK_SIZE(char, &elemP->cdata) - 1;
+    return XMLRPC_MEMBLOCK_SIZE(char, elemP->cdataP) - 1;
 }
 
 
 
-char *
-xml_element_cdata(xml_element * const elemP) {
+const char *
+xml_element_cdata(const xml_element * const elemP) {
     XMLRPC_ASSERT_ELEM_OK(elemP);
-    return XMLRPC_TYPED_MEM_BLOCK_CONTENTS(char, &elemP->cdata);
+    return XMLRPC_MEMBLOCK_CONTENTS(char, elemP->cdataP);
 }
 
 
 
-size_t
+unsigned int
 xml_element_children_size(const xml_element * const elemP) {
     XMLRPC_ASSERT_ELEM_OK(elemP);
-    return XMLRPC_TYPED_MEM_BLOCK_SIZE(xml_element *, &elemP->children);
+    return XMLRPC_MEMBLOCK_SIZE(xml_element *, elemP->childrenP);
 }
 
 
@@ -221,7 +209,7 @@ xml_element_children_size(const xml_element * const elemP) {
 xml_element **
 xml_element_children(const xml_element * const elemP) {
     XMLRPC_ASSERT_ELEM_OK(elemP);
-    return XMLRPC_TYPED_MEM_BLOCK_CONTENTS(xml_element *, &elemP->children);
+    return XMLRPC_MEMBLOCK_CONTENTS(xml_element *, elemP->childrenP);
 }
 
 
@@ -238,9 +226,9 @@ xmlElementAppendCdata(xmlrpc_env *  const envP,
 				      size_t        const size) {
 
     XMLRPC_ASSERT_ENV_OK(envP);
-    XMLRPC_ASSERT_ELEM_OK(elemP);    
+    XMLRPC_ASSERT_ELEM_OK(elemP);
 
-    XMLRPC_TYPED_MEM_BLOCK_APPEND(char, envP, &elemP->cdata, cdata, size);
+    XMLRPC_MEMBLOCK_APPEND(char, envP, elemP->cdataP, cdata, size);
 }
 
 
@@ -260,8 +248,7 @@ xmlElementAppendChild(xmlrpc_env *  const envP,
     XMLRPC_ASSERT_ELEM_OK(childP);
     assert(childP->parentP == NULL);
 
-    XMLRPC_TYPED_MEM_BLOCK_APPEND(xml_element *, envP, &elemP->children,
-                                  &childP, 1);
+    XMLRPC_MEMBLOCK_APPEND(xml_element *, envP, elemP->childrenP, &childP, 1);
     if (!envP->fault_occurred)
         childP->parentP = elemP;
     else
@@ -321,7 +308,7 @@ startElement_(void *           const userData,
             XMLRPC_FAIL_IF_FAULT(&contextP->env);
             contextP->currentP = newCurrentP;
         }
-        
+
     cleanup:
         if (elemP)
             xml_element_free(elemP);
@@ -365,12 +352,12 @@ characterData(void *          const userData,
     ParseContext * contextP;
 
     assert(userData != NULL && s != NULL);
-    
+
     /* Get our context and see if an error has already occured. */
     contextP = (ParseContext*)userData;
     if (!contextP->env.fault_occurred) {
         assert(contextP->currentP != NULL);
-	
+
         xmlElementAppendCdata(&contextP->env,
                               contextP->currentP,
                               (char *)s,
@@ -415,8 +402,8 @@ static xmlSAXHandler const saxHandler = {
     NULL,      /* cdataBlock */
     NULL,      /* externalSubset */
     1          /* initialized */
-    
-    /* Following are SAX2 fields. Any ifdef here? */ 
+
+    /* Following are SAX2 fields. Any ifdef here? */
 
     ,NULL,     /* _private */
     NULL,      /* startElementNs */
@@ -426,12 +413,62 @@ static xmlSAXHandler const saxHandler = {
 
 
 
-void
-xml_parse(xmlrpc_env *   const envP,
-          const char *   const xmlData,
-          size_t         const xmlDataLen,
-          xml_element ** const resultPP) {
+static void
+removeDocSizeLimit(xmlParserCtxt * const parserP ATTR_UNUSED) {
+/*----------------------------------------------------------------------------
+   Set up *parserP to accept a document of any size.
 
+   Newer Libxml2 by default fails any attempt to parse a document larger than
+   10 MiB, because it wants to avoid running the system out of resources.
+   This is not an appropriate role for the library, and 10 MiB is a completely
+   arbitrary number anyway, so we don't use this feature.
+-----------------------------------------------------------------------------*/
+#if LIBXML_VERSION > 20700
+
+    xmlCtxtUseOptions(parserP, XML_PARSE_HUGE);
+
+#else
+
+    /* There's never a size limit */
+
+#endif
+}
+
+
+
+static void
+createParser(xmlrpc_env *     const envP,
+             ParseContext *   const contextP,
+             xmlParserCtxt ** const parserPP) {
+/*----------------------------------------------------------------------------
+   Create an appropriate Libxml2 parser for our purpose.
+-----------------------------------------------------------------------------*/
+    xmlParserCtxt * parserP;
+
+    parserP = xmlCreatePushParserCtxt((xmlSAXHandler *)&saxHandler, contextP,
+                                        NULL, 0, NULL);
+
+    if (!parserP)
+        xmlrpc_faultf(envP, "Failed to create libxml2 parser.");
+    else {
+        removeDocSizeLimit(parserP);
+
+        *parserPP = parserP;
+    }
+}
+
+
+
+void
+xml_parse(xmlrpc_env *      const envP,
+          const char *      const xmlData,
+          size_t            const xmlDataLen,
+          xmlrpc_mem_pool * const memPoolP ATTR_UNUSED,
+          xml_element **    const resultPP) {
+/*----------------------------------------------------------------------------
+  This is an implementation of the interface declared in xmlparser.h.  This
+  implementation uses Libxml2.
+-----------------------------------------------------------------------------*/
     ParseContext context;
     xmlParserCtxt * parserP;
 
@@ -442,11 +479,9 @@ xml_parse(xmlrpc_env *   const envP,
     context.rootP    = NULL;
     context.currentP = NULL;
 
-    parserP = xmlCreatePushParserCtxt((xmlSAXHandler *)&saxHandler, &context,
-                                      NULL, 0, NULL);
-    if (!parserP)
-        xmlrpc_faultf(envP, "Failed to create libxml2 parser.");
-    else {
+    createParser(envP, &context, &parserP);
+
+    if (!envP->fault_occurred) {
         int rc;
 
         rc = xmlParseChunk(parserP, xmlData, xmlDataLen, 1);
