@@ -55,6 +55,7 @@ static void
 test_value_int(void) { 
 
     xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
     xmlrpc_int32 i;
 
@@ -87,6 +88,17 @@ test_value_int(void) {
     TEST_NO_FAULT(&env);
     TEST(i == 10);
 
+    v = xmlrpc_int_new(&env, (xmlrpc_int32) 25);
+    TEST_NO_FAULT(&env);
+    v2 = xmlrpc_value_new(&env, v);
+    TEST_NO_FAULT(&env);
+    xmlrpc_DECREF(v);
+    TEST(xmlrpc_value_type(v2) == XMLRPC_TYPE_INT);
+    xmlrpc_read_int(&env, v2, &i);
+    TEST_NO_FAULT(&env);
+    TEST(i == 25);
+    xmlrpc_DECREF(v2);
+
     xmlrpc_env_clean(&env);
 }
 
@@ -96,6 +108,7 @@ static void
 test_value_bool(void) {
 
     xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
     xmlrpc_bool b;
 
@@ -122,6 +135,17 @@ test_value_bool(void) {
     TEST_NO_FAULT(&env);
     TEST(!b);
 
+    v = xmlrpc_bool_new(&env, (xmlrpc_bool) 1);
+    TEST_NO_FAULT(&env);
+    v2 = xmlrpc_value_new(&env, v);
+    TEST_NO_FAULT(&env);
+    xmlrpc_DECREF(v);
+    TEST(XMLRPC_TYPE_BOOL == xmlrpc_value_type(v2));
+    xmlrpc_read_bool(&env, v2, &b);
+    TEST_NO_FAULT(&env);
+    TEST(b);
+    xmlrpc_DECREF(v2);
+
     xmlrpc_env_clean(&env);
 }
 
@@ -131,6 +155,7 @@ static void
 test_value_double(void) {
 
     xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
     double d;
 
@@ -155,6 +180,31 @@ test_value_double(void) {
     TEST_NO_FAULT(&env);
     TEST(d == 1.0);
 
+    v = xmlrpc_double_new(&env, -3.25);
+    TEST_NO_FAULT(&env);
+    v2 = xmlrpc_value_new(&env, v);
+    xmlrpc_DECREF(v);
+    TEST(XMLRPC_TYPE_DOUBLE == xmlrpc_value_type(v2));
+    xmlrpc_read_double(&env, v2, &d);
+    TEST_NO_FAULT(&env);
+    TEST(d == -3.25);
+    xmlrpc_DECREF(v2);
+
+    /* Older compilers don't have INFINITY and NAN, so we compute them here.
+       But newer compilers recognize an error if they see you dividing by
+       zero, so we have to make it nonobvious that we're doing that.
+    */
+    double const zero = sin(0);
+    
+    v = xmlrpc_double_new(&env, 1.0/zero);  /* +INFINITY */
+    TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
+
+    v = xmlrpc_double_new(&env, -1.0/zero); /* -INFINITY */
+    TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
+
+    v = xmlrpc_double_new(&env, 0.0/zero);  /* NAN */
+    TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
+
     xmlrpc_env_clean(&env);
 }
 
@@ -254,6 +304,21 @@ test_value_string_no_null(void) {
 
     xmlrpc_DECREF(v);
 
+    {
+        const char * const simpleAsciiString = "foo";
+        xmlrpc_value * v2;
+        v = xmlrpc_string_new(&env, simpleAsciiString);
+        TEST_NO_FAULT(&env);
+        v2 = xmlrpc_value_new(&env, v);
+        TEST_NO_FAULT(&env);
+        xmlrpc_DECREF(v);
+        TEST(xmlrpc_value_type(v2) == XMLRPC_TYPE_STRING);
+        xmlrpc_read_string(&env, v2, &str);
+        TEST_NO_FAULT(&env);
+        TEST(streq(str, simpleAsciiString));
+        xmlrpc_DECREF(v2);
+        strfree(str);
+    }        
     xmlrpc_env_clean(&env);
 }
 
@@ -513,6 +578,36 @@ test_value_string_cr(void) {
 }
 
 
+
+static void
+test_value_string_ctlchar(void) {
+
+    xmlrpc_env env;
+    xmlrpc_value * vCtl;
+    xmlrpc_value * vNoCtl;
+
+    xmlrpc_env_init(&env);
+    vNoCtl = xmlrpc_string_new(&env, "foo bar");
+    TEST_NO_FAULT(&env);
+
+    xmlrpc_string_validate(&env, vNoCtl);
+    TEST_NO_FAULT(&env);
+
+    xmlrpc_env_init(&env);
+    vCtl = xmlrpc_string_new(&env, "foo\x18 bar");
+    TEST_NO_FAULT(&env);
+
+    xmlrpc_string_validate(&env, vCtl);
+    TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
+
+    xmlrpc_DECREF(vNoCtl);
+    xmlrpc_DECREF(vCtl);
+
+    xmlrpc_env_clean(&env);
+}
+
+
+
 #if HAVE_UNICODE_WCHAR
 
 /* Here is a 3-character, NUL-terminated string, once in UTF-8 chars,
@@ -629,6 +724,7 @@ test_value_string_wide(void) {
 #if HAVE_UNICODE_WCHAR
     xmlrpc_env env;
     xmlrpc_value * valueP;
+    xmlrpc_value * value2P;
     const wchar_t * wcs;
     size_t len;
 
@@ -731,6 +827,18 @@ test_value_string_wide(void) {
     xmlrpc_read_string_w(&env, valueP, &wcs);
     TEST_FAULT(&env, XMLRPC_TYPE_ERROR);
 
+    value2P = xmlrpc_value_new(&env, valueP);
+    TEST_NO_FAULT(&env);
+
+    xmlrpc_read_string_w_lp(&env, value2P, &len, &wcs);
+    TEST_NO_FAULT(&env);
+    TEST(wcs != NULL);
+    TEST(len == 4);
+    TEST(wcs[len] == '\0');
+    TEST(wcsneq(wcs, wcs_data, len));
+    free((void*)wcs);
+    xmlrpc_DECREF(value2P);
+
     xmlrpc_DECREF(valueP);
 
     test_value_string_wide_line();
@@ -750,6 +858,7 @@ test_value_base64(void) {
     unsigned char const data2[3] = {'a', '\0', 'b'};
 
     xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
     const unsigned char * data;
     size_t len;
@@ -778,6 +887,19 @@ test_value_base64(void) {
     TEST(memeq(data, data1, sizeof(data2)));
     free((void *)data);
 
+    v = xmlrpc_base64_new(&env, sizeof(data1), data1);
+    TEST_NO_FAULT(&env);
+    v2 = xmlrpc_value_new(&env, v);
+    TEST_NO_FAULT(&env);
+    xmlrpc_DECREF(v);
+    TEST(XMLRPC_TYPE_BASE64 == xmlrpc_value_type(v2));
+    xmlrpc_read_base64(&env, v2, &len, &data);
+    TEST_NO_FAULT(&env);
+    TEST(memeq(data, data1, sizeof(data1)));
+    TEST(len == sizeof(data1));
+    xmlrpc_DECREF(v2);
+    free((void*)data);
+
     xmlrpc_env_clean(&env);
 }
 
@@ -813,7 +935,8 @@ test_value_value(void) {
 static void
 test_value_array(void) {
 
-    xmlrpc_value *v;
+    xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
     size_t len;
     xmlrpc_value * itemP;
@@ -840,6 +963,14 @@ test_value_array(void) {
     TEST(len == 1);
     xmlrpc_DECREF(itemP);
 
+    v2 = xmlrpc_value_new(&env, v);
+    TEST_NO_FAULT(&env);
+    TEST(XMLRPC_TYPE_ARRAY == xmlrpc_value_type(v));
+    len = xmlrpc_array_size(&env, v);
+    TEST_NO_FAULT(&env);
+    TEST(len == 1);
+    xmlrpc_DECREF(v2);
+
     xmlrpc_DECREF(v);
 
     v = xmlrpc_build_value(&env, "()");
@@ -1088,6 +1219,7 @@ test_value_cptr(void) {
     int destroyConfirmation;
 
     xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
     void * ptr;
     int myObject;
@@ -1119,6 +1251,13 @@ test_value_cptr(void) {
     TEST_NO_FAULT(&env);
     TEST(ptr == (void*) 0x00000017);
 
+    v = xmlrpc_cptr_new(&env, &myObject);
+    TEST_NO_FAULT(&env);
+    v2 = xmlrpc_value_new(&env, v);
+    xmlrpc_DECREF(v);
+    TEST(xmlrpc_value_type(v2) == XMLRPC_TYPE_C_PTR);
+    xmlrpc_DECREF(v2);
+
     xmlrpc_env_clean(&env);
 }
 
@@ -1128,6 +1267,7 @@ static void
 test_value_nil(void) {
 
     xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
 
     xmlrpc_env_init(&env);
@@ -1146,6 +1286,14 @@ test_value_nil(void) {
     xmlrpc_DECREF(v);
     TEST_NO_FAULT(&env);
 
+    v = xmlrpc_nil_new(&env);
+    TEST_NO_FAULT(&env);
+    v2 = xmlrpc_value_new(&env, v);
+    TEST_NO_FAULT(&env);
+    xmlrpc_DECREF(v);
+    TEST(xmlrpc_value_type(v2) == XMLRPC_TYPE_NIL);
+    xmlrpc_DECREF(v2);
+
     xmlrpc_env_clean(&env);
 }
 
@@ -1155,6 +1303,7 @@ static void
 test_value_i8(void) { 
 
     xmlrpc_value * v;
+    xmlrpc_value * v2;
     xmlrpc_env env;
     xmlrpc_int64 i;
 
@@ -1195,6 +1344,17 @@ test_value_i8(void) {
     TEST_NO_FAULT(&env);
     TEST(i == 10);
 
+    v = xmlrpc_i8_new(&env, (xmlrpc_int64) 25);
+    TEST_NO_FAULT(&env);
+    v2 = xmlrpc_value_new(&env, v);
+    TEST_NO_FAULT(&env);
+    xmlrpc_DECREF(v);
+    TEST(xmlrpc_value_type(v2) == XMLRPC_TYPE_I8);
+    xmlrpc_read_i8(&env, v2, &i);
+    TEST_NO_FAULT(&env);
+    TEST(i == 25);
+    xmlrpc_DECREF(v2);
+
     xmlrpc_env_clean(&env);
 }
 
@@ -1209,7 +1369,8 @@ test_value_type_mismatch(void) {
     char * str;
 
     /* Test for one, simple kind of type mismatch error. We assume that
-    ** if one of these typechecks works, the rest work fine. */
+       if one of these typechecks works, the rest work fine.
+    */
 
     xmlrpc_env_init(&env);
     xmlrpc_env_init(&env2);
@@ -1229,14 +1390,13 @@ test_value_type_mismatch(void) {
 static void
 test_value_invalid_type(void) {
 
-    xmlrpc_value * v;
     xmlrpc_env env;
 
     /* Test invalid type specifier in format string */
 
     xmlrpc_env_init(&env);
 
-    v = xmlrpc_build_value(&env, "Q");
+    xmlrpc_build_value(&env, "Q");
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
     xmlrpc_env_clean(&env);
@@ -1247,17 +1407,16 @@ test_value_invalid_type(void) {
 static void
 test_value_missing_array_delim(void) {
 
-    xmlrpc_value * v;
     xmlrpc_env env;
 
     /* Test missing close parenthesis on array */
 
     xmlrpc_env_init(&env);
 
-    v = xmlrpc_build_value(&env, "(");
+    xmlrpc_build_value(&env, "(");
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
-    v = xmlrpc_build_value(&env, "(i");
+    xmlrpc_build_value(&env, "(i");
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
     xmlrpc_env_clean(&env);
@@ -1268,20 +1427,19 @@ test_value_missing_array_delim(void) {
 static void
 test_value_missing_struct_delim(void) {
 
-    xmlrpc_value * v;
     xmlrpc_env env;
     
     /* Test missing closing brace on struct */
 
     xmlrpc_env_init(&env);
 
-    v = xmlrpc_build_value(&env, "{");
+    xmlrpc_build_value(&env, "{");
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
-    v = xmlrpc_build_value(&env, "{s:i", "key1", 7);
+    xmlrpc_build_value(&env, "{s:i", "key1", 7);
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
-    v = xmlrpc_build_value(&env, "{s:i,s:i", "key1", 9, "key2", -4);
+    xmlrpc_build_value(&env, "{s:i,s:i", "key1", 9, "key2", -4);
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
     xmlrpc_env_clean(&env);
@@ -1292,7 +1450,6 @@ test_value_missing_struct_delim(void) {
 static void
 test_value_invalid_struct(void) {
 
-    xmlrpc_value * v;
     xmlrpc_env env;
 
     /* Note that even though the format strings are invalid, we have
@@ -1303,13 +1460,13 @@ test_value_invalid_struct(void) {
     
     xmlrpc_env_init(&env);
 
-    v = xmlrpc_build_value(&env, "{s:ii", "key1", 9, 9);
+    xmlrpc_build_value(&env, "{s:ii", "key1", 9, 9);
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
-    v = xmlrpc_build_value(&env, "{si:", "key1", 9);
+    xmlrpc_build_value(&env, "{si:", "key1", 9);
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
-    v = xmlrpc_build_value(&env, "{s", "key1");
+    xmlrpc_build_value(&env, "{s", "key1");
     TEST_FAULT(&env, XMLRPC_INTERNAL_ERROR);
 
     xmlrpc_env_clean(&env);
@@ -1349,6 +1506,8 @@ test_value_parse_value(void) {
 
     xmlrpc_env env;
     xmlrpc_value * valueP;
+    const char * base64_data = "base64 data";
+    size_t base64_data_length = strlen(base64_data);
     const char * datestring = "19980717T14:08:55";
 
     xmlrpc_env_init(&env);
@@ -1356,7 +1515,7 @@ test_value_parse_value(void) {
     valueP = xmlrpc_build_value(&env, "(idb8ss#6(i){s:i}np(i))",
                                 7, 3.14, (xmlrpc_bool)1, datestring,
                                 "hello world", "a\0b", (size_t)3,
-                                "base64 data", strlen("base64 data"),
+                                base64_data, base64_data_length,
                                 15, "member9", 9, &valueP, -5);
     
     TEST_NO_FAULT(&env);
@@ -1853,11 +2012,14 @@ test_value(void) {
     test_value_bool();
     test_value_double();
     test_value_datetime();
+    printf("\n  Running string value tests.");
     test_value_string_no_null();
     test_value_string_null();
     test_value_string_multiline();
     test_value_string_cr();
     test_value_string_wide();
+    test_value_string_ctlchar();
+    printf("\n  String value tests done.\n");
     test_value_base64();
     test_value_array();
     test_value_array2();
